@@ -5,8 +5,8 @@ For hashing the geometry of stations, use blake2b with a digest_size of 12 of th
 Similar for the dataset_id, except that the first 8 fields (starting with feature) of the dataset dict should be used for the hashing.
 """
 from datetime import datetime, date
-from typing import List, Optional, Dict, Union
-from pydantic import BaseModel, Field, HttpUrl, conint
+from typing import List, Optional, Dict, Union, Literal
+from pydantic import BaseModel, Field, HttpUrl, conint, confloat
 import orjson
 from enum import Enum
 from tethys_data_models.utils import orjson_dumps
@@ -27,16 +27,16 @@ class ResultType(str, Enum):
 
 
 class TimeSeriesDims(BaseModel):
+    time: int
     geometry: int
     height: int
-    time: int
 
 
 class GridDims(BaseModel):
+    time: int
     lat: int
     lon: int
     height: int
-    time: int
 
 
 class TrajectoryDims(BaseModel):
@@ -44,17 +44,17 @@ class TrajectoryDims(BaseModel):
 
 
 class TimeSeriesBandsDims(BaseModel):
+    time: int
     geometry: int
     height: int
-    time: int
     band: int
 
 
 class GridBandsDims(BaseModel):
+    time: int
     lat: int
     lon: int
     height: int
-    time: int
     band: int
 
 
@@ -65,6 +65,15 @@ class TrajectoryBandsDims(BaseModel):
 
 ResultDims = Union[TimeSeriesDims, GridDims, TrajectoryDims, TimeSeriesBandsDims, GridBandsDims, TrajectoryBandsDims]
 
+result_type_dict = {
+    'time_series': TimeSeriesDims,
+    'grid': GridDims,
+    'trajectory': TrajectoryDims,
+    'time_series_bands': TimeSeriesBandsDims,
+    'grid_bands': GridBandsDims,
+    'trajectory_bands': TrajectoryBandsDims,
+    }
+
 
 class TimeRange(BaseModel):
     """
@@ -94,43 +103,71 @@ class Stats(BaseModel):
 
 
 class ChunkParams(BaseModel):
-    geometry_count: conint(gt=0) = Field(..., description='The number of geometries that should be chunked together. If the result_type is grid, then the geometry_count must be the square of another number. I.e. it must represent the area calc of a square (length * length).')
-    time_interval: conint(gt=0) = Field(None, description='The chunk time interval is the number of days the time chunk should cover (e.g. 365 would be about a year). A value of None indicates that there is no chunking along the time axis.')
-    height_interval: conint(gt=0) = Field(None, description='The chunk height interval is the interval of height indeces that the height should be chunked. A value of None indicates that there is no chunking along the height axis.')
+    block_length: confloat(gt=0) = Field(None, description='The length in decimal degrees of the sides of the block/square for the spatial grouping. A value of None indicates that all geometries or lat/lon combos are split individually.')
+    time_interval: conint(gt=0) = Field(None, description='The chunk time interval is the number of days the time chunk should cover (e.g. 7 would be a week). A value of None indicates that there is no chunking along the time axis.')
+    # height_index_interval: conint(gt=0) = Field(None, description='The chunk height interval is the interval of height indeces that the height should be chunked. A value of None indicates that there is no chunking along the height axis.')
+    # band_index_interval: conint(gt=0) = Field(None, description='The chunk band interval is the interval of band indeces that the height should be chunked. A value of None indicates that there is no chunking along the band axis.')
 
 
 class ResultChunk(BaseModel):
     """
-    Contain the data about a chunk object of a specific result.
+    Contain the data about a chunk object of a result.
     """
-    chunk_id: str
-    key: str
-    bucket: str
+    chunk_id: str = Field(..., description='The chunk id that is unique within this station.')
+    key: str = Field(..., description='The S3 key where this results chunk is stored. This will be totally unique for this results chunk.')
+    # bucket: str
     content_length: conint(gt=0)
-    etag: str
-    heights: Union[List[float], List[int]]
-    time_range: TimeRange = Field(..., description='The maximum time range of the result chunk.')
+    # etag: str
+    # obj_id: str = Field(..., description='This is the version id given by the S3 API response when using a put_object call.')
+    version_date: datetime = Field(..., description='The date that uniquely defines this results version of the dataset and station.')
+    chunk_hash: str = Field(..., description='The hash of the results data stored in the chunk.')
+    dataset_id: str = Field(..., description='The dataset uuid.')
+    station_id: str = Field(..., description='station id based on the geometry.')
+    height: int = Field(None, description='The height multiplied by 1000 (so that it is in mm). Should be omitted if the results does not have height as part of the dimensions.')
+    chunk_day: conint(ge=-106751, le=106751) = Field(None, description='The start day of the interval for this chunk. The chunk day is the number of days after 1970-01-01. Can be negative for days before 1970-01-01 with a minimum of -106751, which is 1677-09-22 (minimum possible date). The maximum value is 106751. Should be omitted if the chunk should include all times or if time is not part of the dimensions.')
+    band: conint(ge=0) = Field(None, description='The band (starting with 0) of the bands array of the results. Should be omitted if the results does not have band as part of the dimensions.')
+
+    class Config:
+        json_loads = orjson.loads
+        json_dumps = orjson_dumps
 
 
 class ResultVersion(BaseModel):
     """
-    Groups the result chunks into an object. Contains information about a specific result version.
+    Contains information about results versions for a dataset.
     """
+    dataset_id: str = Field(..., description='The dataset uuid.')
     version_date: datetime = Field(..., description='The date that uniquely defines this results version of the dataset and station.')
     name: str = None
-    content_length: conint(gt=0)
-    doi: str = None
+    doi: HttpUrl = Field(None, description='The digital object identifier (DOI) for this specific version of the dataset. This should be in the form of an http URL.')
     description: str = Field(None, description='Description of the results version.')
-    result_chunks: List[ResultChunk]
+    modified_date: datetime = Field(..., description='The modification date of the last edit.')
+
+    class Config:
+        json_loads = orjson.loads
+        json_dumps = orjson_dumps
+
+
+# class ResultVersionGroup(BaseModel):
+#     """
+#     Groups many result versions together with the dataset_id and station_id. This is to be saved to the *.results_versions.json.zst object with all of the other stations.
+#     """
+#     dataset_id: str = Field(..., description='The dataset uuid.')
+#     station_id: str = Field(..., description='station id based on the geometry')
+#     result_versions: List[ResultVersion]
+#
+#     class Config:
+#         json_loads = orjson.loads
+#         json_dumps = orjson_dumps
 
 
 class ResultVersionGroup(BaseModel):
     """
-    Groups many result versions together with the dataset_id and station_id. This is to be saved to the *.results_versions.json.zst object with all of the other stations.
+    Groups many result versions and chunks together with the dataset_id. This is to be saved to the *.results_versions.json.zst object with all of the other stations.
     """
-    dataset_id: str = Field(..., description='The dataset uuid.')
-    station_id: str = Field(..., description='station id based on the geometry')
-    result_versions: List[ResultVersion]
+    # dataset_id: str = Field(..., description='The dataset uuid.')
+    results_versions: List[ResultVersion]
+    results_chunks: List[ResultChunk]
 
     class Config:
         json_loads = orjson.loads
@@ -145,6 +182,7 @@ class StationBase(base.Station):
     dimensions: ResultDims
     heights: Union[List[float], List[int]]
     time_range: TimeRange = Field(..., description='The maximum time range of the result.')
+    bands: List[int] = None
     stats: Stats
 
 
@@ -152,7 +190,7 @@ class Station(StationBase):
     """
     Contains the complete station data.
     """
-    result_versions: List[ResultVersion]
+    results_chunks: List[ResultChunk]
     modified_date: datetime = Field(..., description='The modification date of the last edit.')
 
 
@@ -160,7 +198,6 @@ class StationAgg(StationBase):
     """
     Contains the complete station data, but only includes the most recent results version. This object is meant to be combined with all of the other stations in a dataset (i.e. the *.stations.json.zst file).
     """
-    result_versions: ResultVersion
     modified_date: datetime = Field(..., description='The modification date of the last edit.')
 
 
@@ -183,59 +220,84 @@ class Dataset(DatasetBase):
     Full dataset metadata schema.
     """
     dataset_id: str = Field(..., description='The unique dataset id based on the felds in the DatasetBase model.')
-    units: str = Field(..., description='The units of the result.')
+    units: str = Field(..., description='The units of the results.')
     license: str = Field(..., description='The legal data license associated with the dataset defined by the owner.')
     attribution: str = Field(..., description='The legally required attribution text to be distributed with the data defined by the owner.')
     result_type: str = Field(..., description='This describes how the results are structurally stored.')
     # spatial_distribution: str = Field(..., description='This describes how the spatial data are distributed. Either sparse or grid.')
-    geometry_type: str = Field(..., description='This describes how the spatial dimensions are stored. Point, Line, Polygon, or Collection. Follows the OGC spatial data types.')
-    grouping: str = Field(..., description='This describes how the staions and the associated data are grouped. Either none or blocks.')
+    # geometry_type: str = Field(..., description='This describes how the spatial dimensions are stored. Point, Line, Polygon, or Collection. Follows the OGC spatial data types.')
+    # spatial_grouping: str = Field(..., description='This describes how the staions and the associated data are grouped. Either none or blocks.')
     extent: geometry = Field(None, description='The geographical extent of the datset as a simple rectangular polygon.')
     time_range: TimeRange = Field(None, description='The maximum time range of the dataset.')
-    spatial_resolution: float = Field(None, description='The spatial resolution in decimal degrees if the spatial_distribution is grid.')
+    spatial_resolution: float = Field(None, description='The spatial resolution in decimal degrees if the result_type is grid.')
     heights: Union[List[float], List[int]] = Field(None, description='The heights from all available results in the dataset.')
     cf_standard_name: str = Field(None, description='The CF conventions standard name for the parameter.')
     wrf_standard_name: str = Field(None, description='The WRF standard name for the parameter.')
-    precision: float = Field(None, description='The decimal precision of the result values.')
+    precision: float = Field(..., description='The decimal precision of the result values.')
     description: str = Field(None, description='Dataset description.')
     product_description: str = Field(None, description='Overall description of the product if method is simulation.')
-    processing_code: conint(gt=0, lt=7) = Field(None, description='The processing code to determine how the input data should be processed.')
+    # processing_code: conint(gt=0, lt=7) = Field(None, description='The processing code to determine how the input data should be processed.')
     parent_datasets: List[str] = Field(None, description='The parent datasets that this dataset was derived from.')
     properties: Dict = Field(None, description='Any additional dataset specific properties.')
     modified_date: datetime = Field(None, description='The modification date of the last edit.')
-    version: conint(gt=1) = Field(None, description='The version of the metadata structure.')
-    chunk_parameters: ChunkParams
+    system_version: conint(gt=1) = Field(None, description='The version of the metadata structure.')
+    bands: List[conint(ge=0)] = None
+    chunk_parameters: ChunkParams = None
 
     class Config:
         json_loads = orjson.loads
         json_dumps = orjson_dumps
 
 
-class ResultAttrs(BaseModel):
+class ParameterAttrs(DatasetBase):
     """
-    The result attributes that should be in the results netcdf file.
+
     """
-    result_type: ResultType
-    title: str
-    insttution: str
-    license: str
-    source: str
-    history: str
-    version: int
+    dataset_id: str = Field(..., description='The unique dataset id based on the felds in the DatasetBase model.')
+    units: str = Field(..., description='The units of the results.')
+    license: str = Field(..., description='The legal data license associated with the dataset defined by the owner.')
+    attribution: str = Field(..., description='The legally required attribution text to be distributed with the data defined by the owner.')
+    result_type: str = Field(..., description='This describes how the results are structurally stored.')
+    cf_standard_name: str = Field(None, description='The CF conventions standard name for the parameter.')
+    wrf_standard_name: str = Field(None, description='The WRF standard name for the parameter.')
+    precision: float = Field(None, description='The decimal precision of the result values.')
+    description: str = Field(None, description='Dataset description.')
+
+
+# class ResultAttrs(BaseModel):
+#     """
+#     The result attributes that should be in the results netcdf file.
+#     """
+#     result_type: ResultType
+#     title: str
+#     institution: str
+#     license: str
+#     source: str
+#     history: str
+#     version: int
 
 
 class ChunkID(BaseModel):
     """
     Model to define the hashing dict to create the chunk_id.
     """
-    heights_index: List[conint(ge=0)] = Field(None, description='The index locations (starting with 0) of the heights array of the results. Should be omitted if the chunk should include all heights.')
-    start_date: conint(ge=-106751, le=106751) = Field(None, description='The start date of the interval for this chunk. The start date is the unix days after 1970-01-01. Can be negative for times before 1970. Should be omitted if the chunk should include all times.')
+    # station_id: str = Field(..., description='station id based on the geometry')
+    height: int = Field(None, description='The height multiplied by 1000 (so that it is in mm). Should be omitted if the results does not have height as part of the dimensions.')
+    chunk_day: conint(ge=-106751, le=106751) = Field(None, description='The start day of the interval for this chunk. The chunk day is the number of days after 1970-01-01. Can be negative for days before 1970-01-01 with a minimum of -106751, which is 1677-09-22 (minimum possible date). The maximum value is 106751. Should be omitted if the chunk should include all times or if time is not part of the dimensions.')
+    band: conint(ge=0) = Field(None, description='The band (starting with 0) of the bands array of the results. Should be omitted if the results does not have band as part of the dimensions.')
 
     class Config:
         json_loads = orjson.loads
         json_dumps = orjson_dumps
 
 
+class ResultsEncoding(BaseModel):
+    """
+    The encoding model for saving xaaray Datasets to netcdf.
+    """
+    scale_factor: float = None
+    dtype: Literal['int8', 'int16', 'int32', 'int64']
+    _FillValue: int = None
 
 
 # class DataVars(BaseModel):
