@@ -23,7 +23,9 @@ key_patterns = {2: {
                     'stations': 'tethys/v2/{dataset_id}/stations.json.zst',
                     'station': 'tethys/v2/{dataset_id}/{station_id}/station.json.zst',
                     'dataset': 'tethys/v2/{dataset_id}/dataset.json.zst',
-                    'results_object_keys': 'tethys/v2/{dataset_id}/results_object_keys.json.zst'
+                    'results_object_keys': 'tethys/v2/{dataset_id}/results_object_keys.json.zst',
+                    'diagnostics': 'tethys/v2/diagnostics/{run_id}.json.zst',
+                    'interim_results': 'tethys/v2/interim/{run_id}/{dataset_id}/{version_date}/{station_id}.{start_date}.nc.zst'
                     },
                 3: {
                     'results': 'tethys/v3/{dataset_id}/{station_id}/{run_date}.results.nc.zst',
@@ -31,16 +33,20 @@ key_patterns = {2: {
                     'stations': 'tethys/v3/{dataset_id}.stations.json.zst',
                     'station': 'tethys/v3/{dataset_id}/{station_id}.station.json.zst',
                     'dataset': 'tethys/v3/{dataset_id}.dataset.json.zst',
-                    'results_object_keys': 'tethys/v3/{dataset_id}.results_object_keys.json.zst'
+                    'results_object_keys': 'tethys/v3/{dataset_id}.results_object_keys.json.zst',
+                    'diagnostics': 'tethys/v3/diagnostics/{run_id}.json.zst',
+                    'interim_results': 'tethys/v3/interim/{run_id}/{dataset_id}/{version_date}/{station_id}.{start_date}.nc.zst'
                     },
                 4: {
-                    'results': 'tethys/v4/{dataset_id}/{station_id}/{version_date}/{chunk_id}.results.nc.zst',
+                    'results': 'tethys/v4/{dataset_id}/{station_id}/{chunk_id}.{version_date}.results.nc.zst',
                     'datasets': 'tethys/v4.datasets.json.zst',
                     'stations': 'tethys/v4/{dataset_id}.stations.json.zst',
                     'station': 'tethys/v4/{dataset_id}/{station_id}.station.json.zst',
                     'dataset': 'tethys/v4/{dataset_id}.dataset.json.zst',
                     'results_versions': 'tethys/v4/{dataset_id}.results_versions.json.zst',
-                    'diagnostics': 'tethys/v4/diagnostics/{project_id}/{run_date}.json.zst'
+                    # 'results_chunks': 'tethys/v4/{dataset_id}.results_chunks.json.zst',
+                    'diagnostics': 'tethys/v4/diagnostics/{run_id}.json.zst',
+                    'interim_results': 'tethys/v4/interim/{run_id}/{dataset_id}/{station_id}.{start_date}.nc.zst'
                     }
                 }
 
